@@ -1,6 +1,6 @@
 package SPOPS;
 
-# $Id: SPOPS.pm,v 1.56 2001/11/25 01:23:37 lachoy Exp $
+# $Id: SPOPS.pm,v 1.59 2001/11/27 12:28:45 lachoy Exp $
 
 use strict;
 use Data::Dumper    qw( Dumper );
@@ -14,8 +14,8 @@ use Storable        qw( store retrieve nstore );
 $SPOPS::AUTOLOAD  = '';
 @SPOPS::ISA       = qw( Exporter Storable );
 @SPOPS::EXPORT_OK = qw( _w _wm DEBUG );
-$SPOPS::VERSION   = '0.53';
-$SPOPS::Revision  = substr(q$Revision: 1.56 $, 10);
+$SPOPS::VERSION   = '0.54';
+$SPOPS::Revision  = substr(q$Revision: 1.59 $, 10);
 
 # Note that switching on DEBUG will generate LOTS of messages, since
 # many SPOPS classes import this constant
@@ -172,21 +172,21 @@ sub clone {
     my ( $self, $p ) = @_;
     my $class = $p->{_class} || ref $self;
     DEBUG() && _w( 1, "Cloning new object of class ($class) from old ",
-                       "object of class (", ref $self, ")" );
+                      "object of class (", ref $self, ")" );
     my %initial_data = ();
 
+    my $id_field = $class->id_field;
+    if ( $id_field ) {
+        $initial_data{ $id_field } = $p->{ $id_field } || $p->{id};
+    }
+
     while ( my ( $k, $v ) = each %{ $self } ) {
+        next unless ( $k );
+        next if ( $id_field and $k eq $id_field );
         $initial_data{ $k } = $p->{ $k } || $v;
     }
 
-    my $cloned = $class->new({ %initial_data, skip_default_values => 1 });
-
-    # Take care of the ID value AFTER the object has been instantiated
-    # so we can rely on its id() method to take care of any funky
-    # assignment needs
-
-    $cloned->id( $p->{id} ) if ( $p->{id} );
-    return $cloned;
+    return $class->new({ %initial_data, skip_default_values => 1 });
 }
 
 
