@@ -1,22 +1,19 @@
 package SPOPS::Key::DBI::Pool;
 
-# $Id: Pool.pm,v 1.9 2001/08/22 10:51:45 lachoy Exp $
+# $Id: Pool.pm,v 1.12 2001/10/12 21:00:26 lachoy Exp $
 
 use strict;
 use SPOPS qw( _w DEBUG );
 
 @SPOPS::Key::DBI::Pool::ISA      = ();
-$SPOPS::Key::DBI::Pool::VERSION  = '1.8';
-$SPOPS::Key::DBI::Pool::Revision = substr(q$Revision: 1.9 $, 10);
+$SPOPS::Key::DBI::Pool::VERSION  = '1.90';
+$SPOPS::Key::DBI::Pool::Revision = substr(q$Revision: 1.12 $, 10);
 
 
 # Ensure only PRE_fetch_id works.
 
 sub post_fetch_id { return undef }
 
-
-# Do the work
-
 sub pre_fetch_id  {
     my ( $class, $p ) = @_;
 
@@ -24,26 +21,27 @@ sub pre_fetch_id  {
     unless ( $pool_sql ) {
         my $msg   = 'Cannot retrieve ID to insert record';
         SPOPS::Error->set({ 
-             user_msg   => $msg, 
+             user_msg   => $msg,
              type       => 'db',
              system_msg => "No SQL specified in the configuration of ($class) using the key 'pool_sql'",
              method     => 'pre_fetch_id' });
         die $msg;
-    }  
+    }
     DEBUG() && _w( 1, "Getting ID with SQL:\n$pool_sql" );
- 
+
     my $params = { sql => $pool_sql, db => $p->{db} };
     my $values = eval { $class->CONFIG->{pool_value} };
     my $quote  = eval { $class->CONFIG->{pool_quote} };
 
     if ( $values ) {
-        my  $value_type = ref $values;
+        my $value_type = ref $values;
         if ( $value_type ne 'ARRAY' and $value_type ) {
             my $msg   = 'Cannot retrieve ID to insert record';
             SPOPS::Error->set({ 
-                 user_msg   => $msg, 
+                 user_msg   => $msg,
                  type       => 'db',
-                 system_msg => "Configuration key 'pool_value' in ($class) must be a scalar or arrayref.",
+                 system_msg => "Configuration key 'pool_value' in ($class)" .
+                               "must be a scalar or arrayref.",
                  method     => 'pre_fetch_id' });
             die $msg;
         }
@@ -59,10 +57,10 @@ sub pre_fetch_id  {
 
     $params->{return} = 'single';
     my $row = eval { SPOPS::SQLInterface->db_select( $params ) };
-    if ( $@ ) { 
+    if ( $@ ) {
         $SPOPS::Error::user_msg = 'Cannot retrieve ID to insert record';
         die $SPOPS::Error::user_msg;
-    }   
+    }
     DEBUG() && _w( 1, "Returned <<$row->[0]>> for ID" );
     return $row->[0];
 }
@@ -107,10 +105,10 @@ SPOPS::Key::DBI::Pool -- Retrieves ID field information from a pool
 
 This module retrieves a value from a pool of key values matched up to
 tables. It is not as fast as IDENTITY fields
-(L<SPOPS::Key::DBI::Identity>, auto_incrementing values or sequences,
-but can be portable among databases and, most importantly, works in a
-replicated environment. It also has the benefit of being fairly simple
-to understand.
+(L<SPOPS::Key::DBI::Identity|SPOPS::Key::DBI::Identity>,
+auto_incrementing values or sequences, but can be portable among
+databases and, most importantly, works in a replicated environment. It
+also has the benefit of being fairly simple to understand.
 
 Currently, the key fetching procedure is implemented via a
 stored procedure for portability among tools in different
